@@ -387,7 +387,7 @@ namespace FSpot.Ciff {
 			
 			// HEAP is the type CCDR is the subtype
 			if (System.Text.Encoding.ASCII.GetString (header, 6, 8) != "HEAPCCDR") 
-				throw new System.Exception ("Invalid Ciff Header Block");
+				throw new ImageFormatException ("Invalid Ciff Header Block");
 			
 			version =  BitConverter.ToUInt32 (header, 14, little);
 			
@@ -416,13 +416,12 @@ namespace FSpot.Ciff {
 				ImageDirectory props = Root.ReadDirectory (Tag.ImageProps);
 				byte [] date = props.ReadEntry (Tag.TimeStamp);
 
-				if (date == null)
-					return base.Date;
-				else 
+				if (date == null) {
 					System.Console.WriteLine ("NO DATE");
+					return base.Date;
+				}
 
-
-				return new CaptureTime (date, little).LocalTime;
+				return new CaptureTime (date, little).LocalTime.ToUniversalTime ();
 			}
 		}
 
@@ -443,13 +442,12 @@ namespace FSpot.Ciff {
 
 		public override System.IO.Stream PixbufStream ()
 		{
-			System.IO.MemoryStream stream = null; 
 			byte [] data = GetEmbeddedJpeg ();
 			
 			if (data != null)
-				stream = new System.IO.MemoryStream (data);
-				
-			return stream;
+				return new System.IO.MemoryStream (data);
+			else	
+				return DCRawFile.RawPixbufStream (path);
 		}
 
 		public override Gdk.Pixbuf Load (int width, int height)
