@@ -1,8 +1,114 @@
 using System;
 using Gtk;
 using System.Threading;
+using Cms;
 
 namespace FSpot {
+	public class SepiaTone : ColorAdjustment {
+		public SepiaTone (Photo photo) : base (photo)
+		{
+		}
+
+		protected override Cms.Profile GenerateProfile ()
+		{
+			return Cms.Profile.CreateAbstract (nsteps,
+							   1.0,
+							   32.0,
+							   0.0,
+							   0.0,
+							   -100.0,
+							   null,
+							   ColorCIExyY.WhitePointFromTemperature (5000),
+							   ColorCIExyY.WhitePointFromTemperature (9934));
+		}
+	}
+
+	public class Desaturate : ColorAdjustment {
+		public Desaturate (Photo photo) : base (photo)
+		{
+		}
+
+		protected override Cms.Profile GenerateProfile ()
+		{
+			return Cms.Profile.CreateAbstract (nsteps,
+							   1.0,
+							   0.0,
+							   0.0,
+							   0.0,
+							   -100.0,
+							   null,
+							   ColorCIExyY.WhitePointFromTemperature (5000),
+							   ColorCIExyY.WhitePointFromTemperature (5000));
+	
+		}
+	}
+
+	public abstract class ColorAdjustment {
+		protected Photo photo;
+		protected Cms.Profile image_profile;
+		protected Cms.Profile destination_profile;
+		protected Cms.Profile adjustment_profile;
+		protected Gdk.Pixbuf image;
+		protected int nsteps = 20;
+		protected Cms.Intent intent = Cms.Intent.Perceptual;
+
+		public Gdk.Pixbuf Pixbuf {
+			get { return image; }
+			set { image = value; }
+		}
+
+		public ColorAdjustment (Photo photo)
+		{
+			this.photo = photo;
+		}
+		
+		public void SetDestination (Cms.Profile profile)
+		{
+			destination_profile = profile;
+		}
+
+		protected abstract Cms.Profile GenerateProfile ();
+
+		public void Adjust ()
+		{
+			bool create_version = photo.DefaultVersionId == Photo.OriginalVersionId;
+			ImageFile img = ImageFile.Create (photo.DefaultVersionPath);
+
+			if (image == null)
+				image = img.Load ();
+			
+			if (image_profile == null)
+				image_profile = img.GetProfile ();
+			
+			if (image_profile == null)
+				image_profile = Cms.Profile.CreateStandardRgb ();
+
+			if (destination_profile == null)
+				destination_profile = image_profile;
+
+			Gdk.Pixbuf final = new Gdk.Pixbuf (Gdk.Colorspace.Rgb,
+							   false, 8,
+							   image.Width, 
+							   image.Height);
+			
+			Cms.Profile adjustment_profile = GenerateProfile ();
+			Cms.Profile [] list;
+			if (adjustment_profile != null)
+				list = new Cms.Profile [] { image_profile, adjustment_profile, destination_profile };
+			else
+				list = new Cms.Profile [] { image_profile, destination_profile };
+
+			Cms.Transform transform = new Cms.Transform (list,
+								     PixbufUtils.PixbufCmsFormat (image),
+								     PixbufUtils.PixbufCmsFormat (final),
+								     intent, 0x0000);
+			
+			PixbufUtils.ColorAdjust (image, final, transform);
+			photo.SaveVersion (final, create_version);
+		}
+	}
+
+
 	public class ColorDialog : GladeDialog {
 		Gdk.Pixbuf ScaledPixbuf;
 		Gdk.Pixbuf AdjustedPixbuf;
@@ -31,6 +137,9 @@ namespace FSpot {
 
 		[Glade.Widget] private Gtk.CheckButton white_check;
 		[Glade.Widget] private Gtk.CheckButton exposure_check;
+
+		[Glade.Widget] Gtk.Button ok_button;
+		[Glade.Widget] Gtk.VBox   control_vbox;
 		
 		private FSpot.PhotoImageView view;
 
@@ -51,6 +160,9 @@ namespace FSpot {
 			if (brightness_scale == null)
 				return;
 			
+			if (AdjustedPixbuf == null)
+				return;
+
 			Cms.Profile display_profile = Cms.Profile.GetScreenProfile (view.Screen);
 			Cms.Profile [] list;
 			
@@ -162,33 +274,34 @@ namespace FSpot {
 				return;
 
 			Console.WriteLine ("Saving....");
-			Photo photo = (Photo)view.Item.Current;
-			bool create_version = photo.DefaultVersionId == Photo.OriginalVersionId;
-			
-			Gdk.Pixbuf orig = view.CompletePixbuf ();
-			Gdk.Pixbuf final = new Gdk.Pixbuf (Gdk.Colorspace.Rgb,
-							   false, 8,
-							   orig.Width, 
-							   orig.Height);
-				
-			Cms.Profile abs = AdjustmentProfile ();
-			
-			// FIXME this shouldn't use the screen as the destination profile.
-			Cms.Profile destination = Cms.Profile.GetScreenProfile (view.Screen);
-			if (destination == null)
-				destination = Cms.Profile.CreateStandardRgb ();
 
-			Cms.Profile [] list = new Cms.Profile [] { image_profile, abs, destination };
-			Cms.Transform transform = new Cms.Transform (list,
-								     PixbufUtils.PixbufCmsFormat (orig),
-								     PixbufUtils.PixbufCmsFormat (final),
-								     Cms.Intent.Perceptual, 0x0000);
-			
-			PixbufUtils.ColorAdjust (orig,
-						 final,
-						 transform);
-			
+			Photo photo = (Photo)view.Item.Current;
 			try {
+				bool create_version = photo.DefaultVersionId == Photo.OriginalVersionId;
+				
+				Gdk.Pixbuf orig = view.CompletePixbuf ();
+				Gdk.Pixbuf final = new Gdk.Pixbuf (Gdk.Colorspace.Rgb,
+								   false, 8,
+								   orig.Width, 
+								   orig.Height);
+				
+				Cms.Profile abs = AdjustmentProfile ();
+				
+				// FIXME this shouldn't use the screen as the destination profile.
+				Cms.Profile destination = Cms.Profile.GetScreenProfile (view.Screen);
+				if (destination == null)
+					destination = Cms.Profile.CreateStandardRgb ();
+				
+				Cms.Profile [] list = new Cms.Profile [] { image_profile, abs, destination };
+				Cms.Transform transform = new Cms.Transform (list,
+									     PixbufUtils.PixbufCmsFormat (orig),
+									     PixbufUtils.PixbufCmsFormat (final),
+									     Cms.Intent.Perceptual, 0x0000);
+				
+				PixbufUtils.ColorAdjust (orig,
+							 final,
+							 transform);
+				
 				photo.SaveVersion (final, create_version);
 				((PhotoQuery)view.Query).Commit (view.Item.Index);
 			} catch (System.Exception e) {
@@ -234,37 +347,41 @@ namespace FSpot {
 
 		private void HandlePhotoChanged (PhotoImageView view)
 		{
-			if (!view.Item.IsValid) {
-				image_profile = null;
-				return;
-			}
+			try {
+				if (!view.Item.IsValid)
+					throw new Exception ("Invalid Image");
 			
-			FSpot.ImageFile img = FSpot.ImageFile.Create (((Photo)view.Item.Current).DefaultVersionPath);
-			
-			image_profile = img.GetProfile ();
-			
-			// FIXME fall back to rgb for now
-			if (image_profile == null)
-				image_profile = Cms.Profile.CreateStandardRgb ();
-			
-			AdjustedPixbuf = img.Load (256, 256);
-			ScaledPixbuf = AdjustedPixbuf.Copy ();			
+				FSpot.ImageFile img = FSpot.ImageFile.Create (((Photo)view.Item.Current).DefaultVersionPath);
 
-#if false
-			Cms.Profile srgb = Cms.Profile.CreateSRgb ();
-			Cms.Profile lab = Cms.Profile.CreateLab ();
-			Cms.Profile [] list = new Cms.Profile [] { srgb, lab };
-			
-			Cms.Transform t = new Cms.Transform (list, 
-							     PixbufUtils.PixbufCmsFormat (AdjustedPixbuf),
-							     PixbufUtils.PixbufCmsFormat (AdjustedPixbuf),
-							     Cms.Intent.Perceptual, 0x0000);
-			
-			PixbufUtils.ColorAdjust (AdjustedPixbuf,
-						 ScaledPixbuf,
-						 t);
-#endif
-			RangeChanged (null, null);
+				try {
+					image_profile = img.GetProfile ();
+				} catch (System.Exception e) {
+					image_profile = null;
+					System.Console.WriteLine (e);
+				}
+
+				// FIXME fall back to rgb for now
+				if (image_profile == null)
+					image_profile = Cms.Profile.CreateStandardRgb ();
+				
+				AdjustedPixbuf = img.Load (256, 256);
+				ScaledPixbuf = AdjustedPixbuf.Copy ();			
+
+				// FIXME handle alpha
+				if (AdjustedPixbuf.HasAlpha)
+					throw new Exception ("Unsupported Alpha Channel");
+
+				control_vbox.Sensitive = true;
+				ok_button.Sensitive = true;
+
+				RangeChanged (null, null);
+			} catch (System.Exception e) {
+				control_vbox.Sensitive = false;
+				ok_button.Sensitive = false;
+				AdjustedPixbuf = null;
+				ScaledPixbuf = null;
+				image_profile = null;
+			}
 		}
 
 		private const double e = 0.0;
