@@ -1,5 +1,6 @@
 using System.IO;
 using System;
+using Mono.Posix;
 
 namespace FSpot {
 	[DBus.Interface ("org.gnome.FSpot.Core")]
@@ -23,11 +24,13 @@ namespace FSpot {
 		private static Db db;
 		static DBus.Connection connection;
 		System.Collections.ArrayList toplevels;
+		const string ServicePath = "org.gnome.FSpot";
+		const string CorePath = "/org/gnome/FSpot/Core";
 
 		public Core ()
 		{
 			toplevels = new System.Collections.ArrayList ();
-					
+			
 			// Load the database, upgrading/creating it as needed
 			string base_directory = FSpot.Global.BaseDirectory;
 			if (! File.Exists (base_directory))
@@ -52,16 +55,48 @@ namespace FSpot {
 
 		public static Core FindInstance ()
 		{
-			DBus.Service service = DBus.Service.Get (Connection, "org.gnome.FSpot");
-			return (Core)service.GetObject (typeof (Core), "/org/gnome/FSpot/Core");
+			DBus.Service service = DBus.Service.Get (Connection, ServicePath);
+			return (Core)service.GetObject (typeof (Core), CorePath);
+		}
+		
+		public void UnregisterServer ()
+		{
+			try { 
+				connection = null;
+				DBus.Service serv = DBus.Service.Get (Connection, ServicePath);
+				serv.UnregisterObject (this);
+			} catch (System.Exception e) {
+				// noop
+				System.Console.WriteLine ("unregister\n{0}\nunregister", e);
+			}
 		}
 
 		public void RegisterServer ()
 		{
-			DBus.Service service = new DBus.Service (Connection, "org.gnome.FSpot");
-			service.RegisterObject (this, "/org/gnome/FSpot/Core");
+			Service.RegisterObject (this, CorePath);
 		}
 		
+		public static void AssertOwnership () {
+			if (Service == null)
+				System.Console.WriteLine ("problems getting service");
+		}
+		
+		private static DBus.Service Service {
+			get {
+				DBus.Service service = null;
+
+				if (service == null) {
+					try {
+						service = DBus.Service.Get (Connection, ServicePath);
+					} catch (System.Exception e) {
+						//stem.Console.WriteLine (e);
+						service = new DBus.Service (Connection, ServicePath);
+					}
+				}				
+				return service;
+			}
+		}
+
 		private class ImportCommand 
 		{
 			string path;
@@ -160,19 +195,45 @@ namespace FSpot {
 					slideview = new SlideView (black, photos);
 					window.Add (slideview);
 				} else {
-					Gtk.Table table = new Gtk.Table (3, 3, false);
+					Gtk.HBox outer = new Gtk.HBox ();
 					Gtk.HBox hbox = new Gtk.HBox ();
-					table.Attach (new Gtk.HBox (), 0, 1, 0, 1);
-					table.Attach (new Gtk.HBox (), 2, 3, 2, 3);
-					table.Attach (hbox, 1, 2, 1, 2);
+					Gtk.VBox vbox = new Gtk.VBox ();
+
+					outer.PackStart (new Gtk.Label (""));
+					outer.PackStart (vbox, false, false, 0);
+					vbox.PackStart (new Gtk.Label (""));
+					vbox.PackStart (hbox, false, false, 0);
 					hbox.PackStart (new Gtk.Image (Gtk.Stock.DialogWarning, Gtk.IconSize.Dialog),
 							false, false, 0);
-					string msg = Mono.Posix.Catalog.GetString ("No matching photos found");
+					outer.PackStart (new Gtk.Label (""));
+
+					string msg;
+					string long_msg;
+
+					if (tag != null) {
+						msg = String.Format (Catalog.GetString ("No photos matching {0} found"), tag.Name);
+						long_msg = String.Format (Catalog.GetString ("The tag \"{0}\" is not applied to any photos. Try adding\n" +
+											     "the tag to some photos or selecting a different tag in the\n" +
+											     "F-Spot preference dialog."), tag.Name);
+					} else {
+						msg = Catalog.GetString ("Search returned no results");
+						long_msg = Catalog.GetString ("The tag F-Spot is looking for does not exist. Try\n" +
+									      "selecting a different tag in the F-Spot preference\n" +
+									      "dialog.");
+					}
+
 					Gtk.Label label = new Gtk.Label (msg);
-					label.LineWrap = true;
 					hbox.PackStart (label, false, false, 0);
-					window.Add (table);
+
+					Gtk.Label long_label = new Gtk.Label (long_msg);
+					long_label.Markup  = String.Format ("<small>{0}</small>", long_msg);
+
+					vbox.PackStart (long_label, false, false, 0);
+					vbox.PackStart (new Gtk.Label (""));
+
+					window.Add (outer);
 					SetStyle (label);
+					SetStyle (long_label);
 					//SetStyle (image);
 				}
 				window.ShowAll ();
@@ -220,7 +281,7 @@ namespace FSpot {
 				// first too.
 				System.Environment.Exit (0);
 			}
-			if (organizer.Window == sender)
+			if (organizer != null && organizer.Window == sender)
 				organizer = null;
 		}
 	}
