@@ -198,6 +198,7 @@ public class Db : IDisposable {
  	ImportStore import_store;
  	MetaStore meta_store;
 	bool empty;
+	string path;
 
 	public TagStore Tags {
 		get { return tag_store; }
@@ -256,15 +257,70 @@ public class Db : IDisposable {
 		get { return sqlite_connection; }
 	}
 
+	private static int GetFileVersion (string path)
+	{
+		using (Stream stream = File.OpenRead (path)) {
+			byte [] data = new byte [15];
+			stream.Read (data, 0, data.Length);
+
+			string magic = System.Text.Encoding.ASCII.GetString (data, 0, data.Length);
+
+			switch (magic) {
+			case "SQLite format 3":
+				return 3;
+			case "** This file co":
+				return 2;
+			default:
+				return -1;
+			}
+		}
+	}
+
+	public string Repair ()
+	{
+		string backup_path = path;
+		int i = 0;
+
+		while (File.Exists (backup_path)) {
+			backup_path = String.Format ("{0}-{1}-{2}{3}",
+						     Path.GetFileNameWithoutExtension (path),
+						     System.DateTime.Now.ToString ("yyyyMMdd"),
+						     i++,
+						     Path.GetExtension (path));
+		}
+		
+		File.Move (path, backup_path);
+		Init (path, true);
+
+		return backup_path;
+	}
+
 	public void Init (string path, bool create_if_missing)
 	{
 		bool new_db = ! File.Exists (path);
+		string version_string = "";
+		this.path = path;
 
 		if (new_db && ! create_if_missing)
 			throw new Exception (path + ": File not found");
 
+		if (! new_db) {
+			int version = Db.GetFileVersion (path);
+			// FIXME: we should probably display and error dialog if the version
+			// is anything other than the one we were built with, but for now at least
+			// use the right version.
+
+			if (version < 2)
+				throw new Exception ("Unsupported database version");
+			
+			version_string = String.Format (",version={0}", version);
+
+			if (version == 2)
+				version_string += ",encoding=UTF-8";
+		}
+		
 		sqlite_connection = new SqliteConnection ();
-		sqlite_connection.ConnectionString = "URI=file:" + path;
+		sqlite_connection.ConnectionString = "URI=file:" + path + version_string;
 
 		sqlite_connection.Open ();
 		
@@ -274,10 +330,14 @@ public class Db : IDisposable {
 		// Update the database schema if necessary
 		FSpot.Database.Updater.Run (this);
 
+		BeginTransaction ();
+
 		tag_store = new TagStore (sqlite_connection, new_db);
 		import_store = new ImportStore (sqlite_connection, new_db);
  		photo_store = new PhotoStore (sqlite_connection, new_db, tag_store);
 		
+		CommitTransaction ();
+
 		empty = new_db;
 	}
 
