@@ -3,6 +3,13 @@ using Gtk;
 using Gnome;
 using System.Collections;
 using System;
+using System.IO;
+
+public class ImportException : System.Exception {
+	public ImportException (string msg) : base (msg)
+	{
+	}
+}
 
 public class FileImportBackend : ImportBackend {
 	PhotoStore store;
@@ -15,7 +22,8 @@ public class FileImportBackend : ImportBackend {
 
 	ArrayList file_paths;
 	ArrayList imported_photos;
-
+	Stack directories;
+	
 	private void AddPath (string path)
 	{
 		if (FSpot.ImageFile.HasLoader (path))
@@ -51,7 +59,7 @@ public class FileImportBackend : ImportBackend {
 		}
 	
 		foreach (System.IO.FileInfo f in files) {
-			if (exiting_entries [f.Name] == null) {
+			if (! exiting_entries.Contains (f.Name)) {
 				AddPath (f.FullName);
 			}
 		}
@@ -67,7 +75,7 @@ public class FileImportBackend : ImportBackend {
 	public override int Prepare ()
 	{
 		if (file_paths != null)
-			throw new Exception ("Busy");
+			throw new ImportException ("Busy");
 
 		file_paths = new ArrayList ();
 
@@ -84,6 +92,7 @@ public class FileImportBackend : ImportBackend {
 		}	
 
 		imported_photos = new ArrayList ();
+		directories = new Stack ();
 
 		return file_paths.Count;
 	}
@@ -104,8 +113,13 @@ public class FileImportBackend : ImportBackend {
 		
 		return dest;
 	}
-
+	
 	public static string ChooseLocation (string path)
+	{
+		return ChooseLocation (path, null);
+	}
+
+	public static string ChooseLocation (string path, Stack created_directories)
 	{
 		string name = System.IO.Path.GetFileName (path);
 		FSpot.ImageFile img = FSpot.ImageFile.Create (path);
@@ -118,9 +132,34 @@ public class FileImportBackend : ImportBackend {
 						 time.Month,
 						 time.Day);
 		
-		if (!System.IO.Directory.Exists (dest_dir))
-			System.IO.Directory.CreateDirectory (dest_dir);
-		
+		if (!System.IO.Directory.Exists (dest_dir)) {
+			System.IO.DirectoryInfo info;
+			// Split dest_dir into constituent parts so we can clean up each individual directory in
+			// event of a cancel.
+			if (created_directories != null) {
+				string [] parts = dest_dir.Split (new char [] {'/'});
+				string nextPath = "";
+				for (int i = 0; i < parts.Length; i++) {
+					if (i == 0)
+						nextPath += parts [i];
+					else
+						nextPath += "/" + parts [i];
+					if (nextPath.Length > 0) {
+						info = new System.IO.DirectoryInfo (nextPath);
+						// only add the directory path if it didn't already exist and we haven't already added it.
+						if (!info.Exists && !created_directories.Contains (nextPath))
+							created_directories.Push (nextPath);
+					}
+				}
+			}
+			
+			info = System.IO.Directory.CreateDirectory (dest_dir);
+		}
+
+		// If the destination we'd like to use is the file itself return that
+		if (Path.Combine (dest_dir, name) == path)
+			return path;
+		 
 		string dest = UniqueName (dest_dir, name);
 		
 		return dest;
@@ -129,24 +168,39 @@ public class FileImportBackend : ImportBackend {
 	public override bool Step (out Photo photo, out Pixbuf thumbnail, out int count)
 	{
 		if (file_paths == null)
-			throw new Exception ("Prepare() was not called");
+			throw new ImportException ("Prepare() was not called");
 
 		if (this.count == file_paths.Count)
-			throw new Exception ("Already finished");
+			throw new ImportException ("Already finished");
 
 		// FIXME Need to get the EXIF info etc.
 		string path = (string) file_paths [this.count];
 		
 		try {
-			if (copy) {
-				string dest = ChooseLocation (path);
+			string dest = path;
+			if (copy)
+				dest = ChooseLocation (path, directories);
+			
+			if (path == dest) {
+				photo = store.Create (path, out thumbnail);
+			} else {
 				System.IO.File.Copy (path, dest);
 				photo = store.Create (dest, path, out thumbnail);
+				
+				try {
+					File.SetAttributes (dest, File.GetAttributes (dest) & ~FileAttributes.ReadOnly);
+					DateTime create = File.GetCreationTime (path);
+					File.SetCreationTime (dest, create);
+					DateTime mod = File.GetLastWriteTime (path);
+					File.SetLastWriteTime (dest, mod);
+				} catch (System.Exception e) {
+					// we don't want an exception here to be fatal.
+				}
+
 				path = dest;
-			} else {
-                photo = store.Create (path, out thumbnail);
-			}
-			
+
+			} 
+
 			if (tags != null) {
 				foreach (Tag t in tags) {
 					photo.AddTag (t);
@@ -170,7 +224,7 @@ public class FileImportBackend : ImportBackend {
 	public override void Cancel ()
 	{
 		if (imported_photos == null)
-			throw new Exception ("Not doing anything");
+			throw new ImportException ("Not doing anything");
 
 		foreach (Photo p in imported_photos) {
 			if (copy) {
@@ -183,17 +237,28 @@ public class FileImportBackend : ImportBackend {
 			
 			store.Remove (p);
 		}
-
-		Finish ();
+		
+		// clean up all the directories we created.
+		if (copy) {
+			string path;
+			System.IO.DirectoryInfo info;
+			while (directories.Count > 0) {
+				path = directories.Pop () as string;
+				info = new System.IO.DirectoryInfo (path);
+				// double check we aren't trying to delete a directory that still contains something!
+				if (info.Exists && info.GetFiles().Length == 0 && info.GetDirectories().Length == 0)
+					info.Delete ();
+			}
+		}
 	}
 
 	public override void Finish ()
 	{
 		if (file_paths == null)
-			throw new Exception ("Not doing anything");
+			throw new ImportException ("Not doing anything");
 
 		file_paths = null;
-
+		
 		foreach (Photo p in imported_photos) {
 			FSpot.ThumbnailGenerator.Default.Request (p.DefaultVersionPath, 0, 256, 256);
 		}
