@@ -183,7 +183,7 @@ namespace FSpot {
 					progress_dialog.Message = Mono.Posix.Catalog.GetString ("Error While Transferring");
 				}
 
-				Gtk.Application.Invoke (target.ToString (), EventArgs.Empty , new System.EventHandler (Show));
+				GnomeUtil.UrlShow (null, target.ToString ());
 			} catch (System.Exception e) {
 				progress_dialog.Message = e.ToString ();
 				progress_dialog.ProgressText = Mono.Posix.Catalog.GetString ("Error Transferring");
@@ -192,15 +192,10 @@ namespace FSpot {
 				// created.
 				if (!dest.IsLocal)
 					System.IO.Directory.Delete (gallery_path, true);
-				Dialog.Destroy ();
+				
+				Gtk.Application.Invoke (delegate { Dialog.Destroy(); });
 			}
 		}
-		
-		private void Show (object sender, EventArgs args)
-		{
-			if (open)
-				GnomeUtil.UrlShow (null, sender.ToString ());
-		}
 
 		private int Progress (Gnome.Vfs.XferProgressInfo info)
 		{
@@ -245,7 +240,13 @@ namespace FSpot {
 		private void HandleResponse (object sender, Gtk.ResponseArgs args)
 		{
 			if (args.ResponseId != Gtk.ResponseType.Ok) {
+				// FIXME this is to work around a bug in gtk+ where
+				// the filesystem events are still listened to when
+				// a FileChooserButton is destroyed but not finalized
+				// and an event comes in that wants to update the child widgets.
 				Dialog.Destroy ();
+				uri_chooser.Dispose ();
+				uri_chooser = null;
 				return;
 			}
 			
