@@ -404,6 +404,9 @@ public class IconView : Gtk.Layout {
 
 		private void Add (int num, bool notify)
 		{
+			if (num == -1)
+				return;
+		       
 			if (this.Contains (num))
 			    return;
 			
@@ -839,9 +842,9 @@ public class IconView : Gtk.Layout {
 		if (DisplayDates) {
 			string date;
 			if (cell_width > 200) {
-				date = photo.Time.ToString ();
+				date = photo.Time.ToLocalTime ().ToString ();
 			} else {
-				date = photo.Time.ToShortDateString ();
+				date = photo.Time.ToLocalTime ().ToShortDateString ();
 			}
 
 			Pango.Layout layout = (Pango.Layout)date_layouts [date];
