@@ -246,8 +246,15 @@ public class MainWindow {
 		info_box = new InfoBox ();
 		info_box.VersionIdChanged += HandleInfoBoxVersionIdChange;
 		left_vbox.PackStart (info_box, false, true, 0);
+		
+		try {
+			query = new FSpot.PhotoQuery (db.Photos);
+		} catch (System.Exception e) {
+			//FIXME assume any exception here is due to a corrupt db and handle that.
+			RestoreDb (e);
+			query = new FSpot.PhotoQuery (db.Photos);
+		}
 
-		query = new FSpot.PhotoQuery (db.Photos);
 		query.Changed += HandleQueryChanged;
 
 		db.Photos.ItemsChanged += HandleDbItemsChanged;
@@ -293,7 +300,7 @@ public class MainWindow {
 		icon_view.DragDataGet += HandleIconViewDragDataGet;
 
 		TagMenu menu = new TagMenu (attach_tag, db.Tags);
-		menu.NewTagHandler = HandleCreateTagAndAttach;
+		menu.NewTagHandler += delegate { HandleCreateTagAndAttach (main_window, null); };
 		menu.TagSelected += HandleAttachTagMenuSelected;
 
 		near_image.SetFromStock ("f-spot-stock_near", IconSize.SmallToolbar);
@@ -446,10 +453,14 @@ public class MainWindow {
 			Photo p = item as Photo;
 			if (p == null)
 				continue;
-			
-			if (write_metadata)
-				p.WriteMetadataToImage ();
-			
+
+			if (write_metadata) {
+				try {
+					p.WriteMetadataToImage ();
+				} catch (System.Exception e) {
+					Console.WriteLine ("Error syncing metadata to file\n{0}", e);
+				}
+			}
 		}
 		
 		if (args is TimeChangedEventArgs)
@@ -478,7 +489,7 @@ public class MainWindow {
 	public int [] SelectedIds () {
 		int [] ids = new int [0];
 
-		if (fsview != null)
+		if (fsview != null && fsview.View.Item.IsValid)
 			ids = new int [] { fsview.View.Item.Index };
 		else {
 			switch (view_mode) {
@@ -668,9 +679,9 @@ public class MainWindow {
 	// Commands
 	//
 
-	private void RotateSelectedPictures (RotateDirection direction)
+	private void RotateSelectedPictures (Gtk.Window parent, RotateDirection direction)
 	{
-		RotateCommand command = new RotateCommand (main_window);
+		RotateCommand command = new RotateCommand (parent);
 		
 		int [] selected_ids = SelectedIds ();
 		if (command.Execute (direction, SelectedPhotos (selected_ids))) {
@@ -727,6 +738,20 @@ public class MainWindow {
 		args.RetVal = true;
 	}
 
+	void RestoreDb (System.Exception e)
+	{
+		string backup = db.Repair ();
+		string short_msg = Mono.Posix.Catalog.GetString ("Error loading database.");
+		string long_msg = Mono.Posix.Catalog.GetString ("F-Spot encountered an error while loading the photo database" + 
+								"  the old database has be moved to {0} and a new database has been created.");
+
+		HigMessageDialog md = new HigMessageDialog (main_window, DialogFlags.DestroyWithParent, 
+							    MessageType.Error, ButtonsType.Ok, 
+							    short_msg, String.Format (long_msg, backup));
+		md.Run ();
+		md.Destroy ();
+	}
+
 	void HandleTagSelectionDragBegin (object sender, DragBeginArgs args)
 	{
 		Tag [] tags = tag_selection_widget.TagHighlight;
@@ -1400,7 +1425,7 @@ public class MainWindow {
 		StringBuilder url = new StringBuilder ("mailto:?subject=my%20photos");
 
 		foreach (Photo p in SelectedPhotos ()) {
-			url.Append ("&attach=" + HttpUtility.UrlEncode(p.DefaultVersionPath));
+			url.Append ("&attach=" + UriList.EscapeString (p.DefaultVersionPath, true, true, true));
 		}
 		GnomeUtil.UrlShow (main_window, url.ToString ());
 	}
@@ -1554,7 +1579,7 @@ public class MainWindow {
 	{
 		PhotoVersionCommands.Create cmd = new PhotoVersionCommands.Create ();
 
-		if (cmd.Execute (db.Photos, CurrentPhoto, main_window)) {
+		if (cmd.Execute (db.Photos, CurrentPhoto, GetToplevel (null))) {
 			query.MarkChanged (ActiveIndex ());
 		}
 	}
@@ -1563,7 +1588,7 @@ public class MainWindow {
 	{
 		PhotoVersionCommands.Delete cmd = new PhotoVersionCommands.Delete ();
 
-		if (cmd.Execute (db.Photos, CurrentPhoto, main_window)) {
+		if (cmd.Execute (db.Photos, CurrentPhoto, GetToplevel (null))) {
 			query.MarkChanged (ActiveIndex ());
 		}
 	}
@@ -1594,7 +1619,7 @@ public class MainWindow {
 	
 	public void HandleCreateTagAndAttach (object sender, EventArgs args)
 	{
-		Tag new_tag = CreateTag ();
+		Tag new_tag = CreateTag (sender, args);
 
 		if (new_tag != null)
 			HandleAttachTagMenuSelected (new_tag);
@@ -1602,7 +1627,7 @@ public class MainWindow {
 
 	public void HandleCreateNewCategoryCommand (object sender, EventArgs args)
 	{
-		Tag new_tag = CreateTag ();
+		Tag new_tag = CreateTag (sender, args);
 		
 		if (new_tag != null) {
 			tag_selection_widget.ScrollTo (new_tag);
@@ -1610,9 +1635,9 @@ public class MainWindow {
 		}
 	}
 
-	public Tag CreateTag ()
+	public Tag CreateTag (object sender, EventArgs args)
 	{
-		TagCommands.Create command = new TagCommands.Create (db.Tags, main_window);
+		TagCommands.Create command = new TagCommands.Create (db.Tags, GetToplevel (sender));
 		return command.Execute (TagCommands.TagType.Category, tag_selection_widget.TagHighlight);
 	}
 
@@ -1639,10 +1664,7 @@ public class MainWindow {
 			Photo p = query.Photos [num];
 
 			p.RemoveTag (tags);
-			/*
-			if (write_metadata)
-				p.WriteMetadataToImage ();
-			*/
+
 			query.Commit (num);
 		}
 		db.CommitTransaction ();
@@ -1789,15 +1811,15 @@ public class MainWindow {
 
 		if (response == Gtk.ResponseType.Ok) {
 			foreach (int id in SelectedIds ()) {
-				Gdk.Pixbuf orig = FSpot.PhotoLoader.Load (query, id);
-				Gdk.Pixbuf final = PixbufUtils.UnsharpMask (orig, radius_spin.Value, amount_spin.Value, threshold_spin.Value);
-			
 				Photo photo = query.Photos [id];
-				
-				bool create_version = photo.DefaultVersionId == Photo.OriginalVersionId;
-
 				try {
+					Gdk.Pixbuf orig = FSpot.PhotoLoader.Load (query, id);
+					Gdk.Pixbuf final = PixbufUtils.UnsharpMask (orig, radius_spin.Value, amount_spin.Value, threshold_spin.Value);
+					
+					bool create_version = photo.DefaultVersionId == Photo.OriginalVersionId;
+
 					photo.SaveVersion (final, create_version);
+					query.Commit (id);
 				} catch (System.Exception e) {
 					string msg = Mono.Posix.Catalog.GetString ("Error saving sharpened photo");
 					string desc = String.Format (Mono.Posix.Catalog.GetString ("Received exception \"{0}\". Unable to save photo {1}"),
@@ -2044,7 +2066,22 @@ public class MainWindow {
 			main_window, DialogFlags.DestroyWithParent, MessageType.Error,
 			error, msg, ok_caption);
 	}
-	
+
+	public Gtk.Window GetToplevel (object sender)
+	{
+		Widget wsender = sender as Widget;
+		Gtk.Window toplevel = null;
+
+		if (wsender != null)
+			toplevel = (Gtk.Window) wsender.Toplevel;
+		else if (fsview != null)
+			toplevel = fsview;
+		else 
+			toplevel = main_window;
+
+		return toplevel;
+	}
+
 	public void HandleDeleteCommand (object sender, EventArgs args)
 	{
    		Photo[] photos = SelectedPhotos();
@@ -2056,7 +2093,13 @@ public class MainWindow {
 								 "This deletes all versions of the selected photos from your drive.", 
 								 photos.Length);
 		string ok_caption = Mono.Posix.Catalog.GetPluralString ("_Delete photo", "_Delete photos", photos.Length);
-		if (ResponseType.Ok == HigMessageDialog.RunHigConfirmation(main_window, DialogFlags.DestroyWithParent, MessageType.Warning, header, msg, ok_caption)) {                              
+		
+
+
+		if (ResponseType.Ok == HigMessageDialog.RunHigConfirmation(GetToplevel (sender), 
+									   DialogFlags.DestroyWithParent, 
+									   MessageType.Warning, 
+									   header, msg, ok_caption)) {                              
 			
 			foreach (Photo photo in photos) {
 				foreach (uint id in photo.VersionIds) {
@@ -2084,7 +2127,7 @@ public class MainWindow {
 		header = String.Format (header, photos.Length);
 		string msg = Mono.Posix.Catalog.GetString("If you remove photos from the F-Spot catalog all tag information will be lost. The photos remain on your computer and can be imported into F-Spot again.");
 		string ok_caption = Mono.Posix.Catalog.GetString("_Remove from Catalog");
-		if (ResponseType.Ok == HigMessageDialog.RunHigConfirmation(main_window, DialogFlags.DestroyWithParent, 
+		if (ResponseType.Ok == HigMessageDialog.RunHigConfirmation(GetToplevel (sender), DialogFlags.DestroyWithParent, 
 									   MessageType.Warning, header, msg, ok_caption)) {                              
 			db.Photos.Remove (photos);
 			UpdateQuery ();
@@ -2187,12 +2230,12 @@ public class MainWindow {
 
 	public void HandleRotate90Command (object sender, EventArgs args)
 	{
-		RotateSelectedPictures (RotateDirection.Clockwise);
+		RotateSelectedPictures (GetToplevel (sender), RotateDirection.Clockwise);
 	}
 
 	public void HandleRotate270Command (object sender, EventArgs args)
 	{
-		RotateSelectedPictures (RotateDirection.Counterclockwise);
+		RotateSelectedPictures (GetToplevel (sender), RotateDirection.Counterclockwise);
 	}
 
 	public void HandleCopyLocation (object sender, EventArgs args)
@@ -2490,7 +2533,7 @@ public class MainWindow {
 		}
 	}
 
-	public void HandleOpenWith (Gnome.Vfs.MimeApplication mime_application)
+	public void HandleOpenWith (object sender, Gnome.Vfs.MimeApplication mime_application)
 	{
 		Photo[] selected = SelectedPhotos ();
 
@@ -2504,7 +2547,7 @@ public class MainWindow {
 				selected.Length, mime_application.Name);
 
 		// FIXME add cancel button? add help button?
-		HigMessageDialog hmd = new HigMessageDialog(main_window, DialogFlags.DestroyWithParent, 
+		HigMessageDialog hmd = new HigMessageDialog(GetToplevel (sender), DialogFlags.DestroyWithParent, 
 							    MessageType.Question, Gtk.ButtonsType.None,
 							    header, msg);
 
@@ -2525,17 +2568,28 @@ public class MainWindow {
 
 		bool create_new_versions = (response == Gtk.ResponseType.Yes);
 
+		ArrayList errors = new ArrayList ();
 		GLib.List uri_list = new GLib.List (typeof (string));
 		foreach (Photo photo in selected) {
-			if (create_new_versions) {
-				// FIXME exception handling? Out of space, blah, blah
-				uint version = photo.CreateNamedVersion (mime_application.Name, photo.DefaultVersionId, true);
-				photo.DefaultVersionId = version;
-			}
+			// FIXME need to clean up the error dialog here.
+			try {
+				if (create_new_versions) {
+					uint version = photo.CreateNamedVersion (mime_application.Name, photo.DefaultVersionId, true);
+					photo.DefaultVersionId = version;
+				}
 
-			uri_list.Append (photo.DefaultVersionPath);
+				uri_list.Append (photo.DefaultVersionPath);
+			} catch (Exception e) {
+				errors.Add (new EditException (photo, e));
+			}
 		}
 
+		if (errors.Count > 0) {
+			Dialog md = new EditExceptionDialog (GetToplevel (sender), errors.ToArray (typeof (EditException)) as EditException []);
+			md.Run ();
+			md.Destroy ();
+		}
+		
 		if (create_new_versions) {
 			db.Photos.Commit (selected, new DbItemEventArgs (selected));
 		}
