@@ -265,7 +265,8 @@ class PixbufUtils {
 	}
 
 	[DllImport("libgdk_pixbuf-2.0-0.dll")]
-	static extern bool gdk_pixbuf_save_to_bufferv (IntPtr raw, out IntPtr data, out uint length, string type, 
+	static extern bool gdk_pixbuf_save_to_bufferv (IntPtr raw, out IntPtr data, out IntPtr length, 
+						       string type, 
 						       string [] keys, string [] values, out IntPtr error);
 
 					
@@ -273,7 +274,7 @@ class PixbufUtils {
 	{
 		IntPtr error = IntPtr.Zero;
 		IntPtr data;
-		uint length;
+		IntPtr length;
 		string [] terminated_options = null;
 		string [] terminated_values = null;
 
@@ -287,18 +288,21 @@ class PixbufUtils {
 			Array.Copy (values, terminated_values, values.Length);
 		}
 
-		gdk_pixbuf_save_to_bufferv (pixbuf.Handle, 
-					    out data, 
-					    out length, 
-					    type,
-					    terminated_options,
-					    terminated_values,
-					    out error);
-
+		bool success = gdk_pixbuf_save_to_bufferv (pixbuf.Handle, 
+							     out data, 
+							     out length, 
+							     type,
+							     terminated_options,
+							     terminated_values,
+							     out error);
+		
 		if (error != IntPtr.Zero) 
 			throw new GLib.GException (error);
 
-		byte [] content = new byte [length];
+		if (!success)
+			throw new ApplicationException ("Unknown error while saving file");
+
+		byte [] content = new byte [(int)length];
 		Marshal.Copy (data, content, 0, (int)length);
 
 		return content;
